﻿using System;
using System.Diagnostics;
using System.IO;
using System.Windows.Forms;
using Microsoft.WindowsAPICodePack.Shell;
using Microsoft.WindowsAPICodePack.Taskbar;

namespace PuTTYJL
{
    public partial class SettingsForm : Form
    {
        static Properties.Settings settings = Properties.Settings.Default;
        JumpList jl;

        private static readonly string AppID = "net.taricorp.PuttyJL";

        public SettingsForm()
        {
            InitializeComponent();

            pathBox.Text = Properties.Settings.Default.PuTTYPath;
            sessionSelector.Items.AddRange(readRegistrySessions());
            pruneMissingSessions();
            switch (settings.DefaultLaunch)
            {
                case "/putty":
                    radioButton1.Checked = true;
                    break;
                case "/session":
                    radioButton2.Checked = true;
                    break;
                case "/config":
                    radioButton3.Checked = true;
                    break;
            }
        }

        private void button_Save_Click(object sender, EventArgs e)
        {
            if (File.Exists(pathBox.Text))
            {
                //Save settings
                settings.PuTTYPath = pathBox.Text;
                settings.Save();
                updateWindowStatus("Saved settings.");
                //Refresh jumplist
                rebuildJumplist();
            }
            else
            {
                MessageBox.Show("The given path to PuTTY is invalid.  Specify a file that exists.", "Bad path");
            }
        }

        /// <summary>
        /// Builds and refreshes the jumplist.
        /// </summary>
        private void rebuildJumplist()
        {
            jl = JumpList.CreateJumpList();
            jl.KnownCategoryToDisplay = JumpListKnownCategoryType.Neither;
            //Enabled sessions
            foreach (string session in settings.EnabledSessions)
            {
                jl.AddUserTasks(new JumpListLink(settings.PuTTYPath, session)
                    {
                        IconReference = new IconReference(settings.PuTTYPath, 0),
                        Arguments = String.Format("-load \"{0}\"", session),
                        AppID = AppID
                    });
            }
            //Static entry for PuTTY itself.
            jl.AddUserTasks(new JumpListSeparator());
            jl.AddUserTasks(
                new JumpListLink(settings.PuTTYPath, "PuTTY")
                {
                    IconReference = new IconReference(settings.PuTTYPath, 0),
                    AppID = AppID
                },
                new JumpListLink(Application.ExecutablePath, "Configure PuTTYJL")
                {
                    IconReference = new IconReference(Application.ExecutablePath, 0),
                    Arguments = "/config",
                });
            jl.Refresh();
            updateWindowStatus("Jumplist rebuilt.");
        }

        private void button_Cancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void button_Locate_Click(object sender, EventArgs e)
        {
            FileDialog fd = new OpenFileDialog();
            fd.InitialDirectory = Application.ExecutablePath;
            fd.Filter = "PuTTY|PuTTY.exe|Executables (*.exe)|*.exe|All files (*.*)|*.*";
            fd.CheckFileExists = true;
            fd.Title = "Locate PuTTY";
            fd.ShowDialog();
            string path = fd.FileName;
            if (path.Length != 0)
                pathBox.Text = path;
        }

        private void sessionSelector_IndexChanged(object sender, EventArgs e)
        {
            string ss = (string)sessionSelector.SelectedItem;
            //Set enabled box as needed
            if (settings.EnabledSessions.Contains(ss))
                sessionEnabledCheck.Checked = true;
            else
                sessionEnabledCheck.Checked = false;
            //..default box
            if (settings.DefaultSession == ss)
                defaultSessionCheck.Checked = true;
            else
                defaultSessionCheck.Checked = false;
        }

        private void sessionEnabledCheck_CheckedChanged(object sender, EventArgs e)
        {
            string session = (string)sessionSelector.SelectedItem;
            //Trickiness to only change window status message when the user clicks it.
            if (sessionEnabledCheck.Checked && !settings.EnabledSessions.Contains(session))
                settings.EnabledSessions.Add(session);
            else if (!sessionEnabledCheck.Checked && settings.EnabledSessions.Contains(session))
                settings.EnabledSessions.Remove(session);
            else
                return;
            string enabledStatus = sessionEnabledCheck.Checked ? "Enabled" : "Disabled";
            updateWindowStatus("{0} session \"{1}\"", enabledStatus, session);
        }

        private void defaultSessionCheck_CheckedChanged(object sender, EventArgs e)
        {
            string session = (string)sessionSelector.SelectedItem;
            //Only show status when user clicks, not when we auto-change.
            if (defaultSessionCheck.Checked && settings.DefaultSession != session)
                settings.DefaultSession = session;
            else if (!defaultSessionCheck.Checked && settings.DefaultSession == session)
                settings.DefaultSession = "";
            else
                return;
            updateWindowStatus("{0} default session.", defaultSessionCheck.Checked ? "Set" : "Unset");
        }

        private void radioButton_CheckedChanged(object sender, EventArgs e)
        {
            RadioButton s = (RadioButton)sender;
            //Event gets called on both buttons that get changed.
            if (s.Checked)
                settings.DefaultLaunch = (string)(s.Tag);
        }

        /// <summary>
        /// Reads the list of stored sessions for PuTTY from the registry.
        /// </summary>
        /// <returns>A collection of the session names stored in the user registry.</returns>
        private string[] readRegistrySessions()
        {
            string[] sessionNames = new string[0];
            using (Microsoft.Win32.RegistryKey sessionsKey =
                    Microsoft.Win32.Registry.CurrentUser.OpenSubKey("Software\\SimonTatham\\PuTTY\\Sessions"))
                sessionNames = sessionsKey.GetSubKeyNames();
            updateWindowStatus("Read {0} stored session(s), {1} enabled.", sessionNames.Length, settings.EnabledSessions.Count);
            //Unescape session names
            for (int i = 0; i < sessionNames.Length; i++)
                sessionNames[i] = Uri.UnescapeDataString(sessionNames[i]);
            return sessionNames;
        }

        /// <summary>
        /// Prunes sessions which no longer exist in the registry from the saved lists, then unsets
        /// the default session if it no longer exists.
        /// </summary>
        private void pruneMissingSessions()
        {
            //Saved session prune
            for (int i = settings.EnabledSessions.Count - 1; i >= 0; i--)
            {
                string s = settings.EnabledSessions[i];
                if (!sessionSelector.Items.Contains(s))
                {
                    settings.EnabledSessions.RemoveAt(i);
                    updateWindowStatus("Pruned nonexistant session \"{0}\"", s);
                    settings.Save();
                }
            }
            //Verify default
            if (settings.DefaultSession.Length > 0 && !sessionSelector.Items.Contains(settings.DefaultSession))
            {
                updateWindowStatus("Unset nonexistant default session \"{0}\"", settings.DefaultSession);
                settings.DefaultSession = "";
            }
        }

        /// <summary>
        /// Writes a message to the window status strip.
        /// </summary>
        /// <param name="message">String to write to status strip.</param>
        private void updateWindowStatus(string message)
        {
            toolStripStatusLabel1.Text = message;
        }

        /// <summary>
        /// Writes a formatted string to the window status strip.
        /// </summary>
        /// <param name="format">Format string.</param>
        /// <param name="args">Format arguments.</param>
        private void updateWindowStatus(string format, params object[] args)
        {
            updateWindowStatus(String.Format(format, args));
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            TaskbarManager.Instance.ApplicationId = AppID;

            if (settings.EnabledSessions == null)
                settings.EnabledSessions = new System.Collections.Specialized.StringCollection();
            try
            {
                string action;
                if (args.Length == 0)
                    if (settings.DefaultLaunch.Length > 0)
                        action = settings.DefaultLaunch;
                    else
                        action = "";
                else
                    action = args[0];
                //Do requested action
                switch (action)
                {
                    case "/session":
                        defaultSession();
                        break;
                    case "/putty":
                        launchPutty("");
                        break;
                    case "/config":
                    default:
                        showConfig();
                        break;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(String.Format("Unable to perform default action:\n{0}\n{1}", e.Message, e.StackTrace), "Exception");
                showConfig();
            }
        }

        /// <summary>
        /// Starts PuTTY with the given arguments.
        /// </summary>
        /// <param name="args">String to pass as command-line arguments.</param>
        private static void launchPutty(string args)
        {
            using (Process p = new Process())
            {
                p.StartInfo.FileName = settings.PuTTYPath;
                p.StartInfo.Arguments = args;
                p.Start();
                //Wait for a window to show up, then set its AppID
                //do
                //{
                //    System.Threading.Thread.Sleep(0);
                //    p.Refresh();
                //} while (p.MainWindowHandle == IntPtr.Zero);
                //TaskbarManager.Instance.SetApplicationIdForSpecificWindow(p.MainWindowHandle, AppID);
            }
        }

        /// <summary>
        /// Launch the default PuTTY session.
        /// </summary>
        private static void defaultSession()
        {
            launchPutty(String.Format("-load \"{0}\"", settings.DefaultSession));
        }

        /// <summary>
        /// Show the config dialog.
        /// </summary>
        private static void showConfig()
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new SettingsForm());
        }
    }
}
